/*--[drm5comm.c]---------------------------------------------------------------
 | Copyright (C) 2002, 2003 Dan A. Jackson 
 |
 | This file is part of the "clit" (Convert LIT) program. 
 |
 | Convert LIT is free software; you can redistribute it and/or modify
 | it under the terms of the GNU General Public License as published by
 | the Free Software Foundation; either version 2 of the License, or
 | (at your option) any later version.
 |
 | This program is distributed in the hope that it will be useful,
 | but WITHOUT ANY WARRANTY; without even the implied warranty of
 | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 | GNU General Public License for more details.
 |
 | You should have received a copy of the GNU General Public License
 | along with this program; if not, write to the Free Software
 | Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 | 
 | The GNU General Public License may also be available at the following
 | URL: http://www.gnu.org/licenses/gpl.html
 */
#include "drm5hook.h"

HANDLE  hMap, hExec, hDone;
BYTE    *buffer; 
HMODULE  hMyself = 0;

p_external_error_handler    error_handler;

#define ERR_BUFSZ  3*1024
static char error_buffer[ERR_BUFSZ + 1]; 

void common_error_handler(int isSystem, char * fmt, ...)
{
    int ofs, len;
    va_list ap; 
    DWORD   error_code;

    va_start(ap, fmt);
    ofs = FormatMessage(FORMAT_MESSAGE_FROM_STRING, fmt, 0,
        MAKELANGID(LANG_NEUTRAL,SUBLANG_DEFAULT),
        (void *)error_buffer,ERR_BUFSZ, &ap);
    len = ERR_BUFSZ - ofs;
    if (isSystem > 0) {
        error_code = GetLastError();
        if (len > 0)
        {
            ofs += FormatMessage(FORMAT_MESSAGE_FROM_STRING|
                FORMAT_MESSAGE_ARGUMENT_ARRAY,"\nError Code: %1!d!\n", 0, 
                MAKELANGID(LANG_NEUTRAL,SUBLANG_DEFAULT),
                (void *)(error_buffer+ofs), len, (char **)&error_code);
            len = ERR_BUFSZ - ofs; 
        }
        if (len > 0)
        {
            ofs += FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM|
                FORMAT_MESSAGE_IGNORE_INSERTS,NULL, error_code, 
                MAKELANGID(LANG_NEUTRAL,SUBLANG_DEFAULT),
                (void *)(error_buffer + ofs), len, NULL);
        }
    }
    error_buffer[ERR_BUFSZ] = '\0';
    if ((error_handler) && (isSystem >= 0)) {
        error_handler(0, error_buffer);  
    } else {
        MessageBoxA(NULL,error_buffer,"DRM5 HOOK DLL - ERROR", 
            MB_ICONERROR|MB_OK);
    }
}

void common_destroy_global_state(void)
{
    if (buffer) UnmapViewOfFile((LPVOID)buffer);
    buffer = NULL;
    if (hMap) CloseHandle(hMap);
    if (hExec) CloseHandle(hExec);
    if (hDone) CloseHandle(hDone);
    hMap = hExec = hDone = 0;
}

extern BOOL WINAPI _DllMainCRTStartup( HINSTANCE hinst, DWORD reason, LPVOID s) 
{
    if (reason == DLL_PROCESS_ATTACH)
    {
        hMyself = hinst;
        DisableThreadLibraryCalls(hinst); 
    }
    else if (reason == DLL_PROCESS_DETACH) 
    {
        common_destroy_global_state();
    }
    return TRUE;
}
